//-----------------------------------------------------------------------------
//
//						XMM Science Analysis System (SAS)
//					   (c) 1997-2010 European Space Agency
//
//-----------------------------------------------------------------------------
//
//	EarthPositionLocator.cc	--- implementation of EarthPositionLocator classes
//
//	DESCRIPTION:
//		Implementation part of EarthPositionLocator classes
//
//	CHANGE HISTORY:
//		UL, 2000-11-02: 1.0.0: - first version
//		UL, 2000-11-06: 1.1.0: - fixed units problem with altitude
//		UL, 2001-02-26: 1.2.0: - bug fix in EarthObject constructor
//		UL, 2001-12-04: 1.2.1: - siderial -> sidereal
//
//	AUTHOR:
//		U. Lammers, Wed Aug 12 14:02:33 MET DST 1998
//
//	REFERENCES:
//		[1]: Software Specification Document for the XMM Science Analysis
//			 Subsystem, M. Dahlem et. al., 
//		[2]: "Fundamental Astronomy", H. Karttunen et al., Springer-Verlag
//
//	$Header: /sasbuild/repositories/sasdev/sas/packages/caloalutils/src/EarthPositionLocator.cc,v 1.7 2004/08/04 12:18:14 rsaxton Exp $
//

#include <cmath>

#include "EarthPositionLocator.h"

//
//	---( EarthObject )--------------------------------------------------------
//
EarthObject::EarthObject(double longitude,
						 double lattitude, LattitudeType lattType,
						 double altitude, const std::string &name)
	throw(BadLattitude)
	: _long(fmod(longitude, M_TWO_PI)), _altitude(1.e-3*altitude), // m -> km
	  _name(name)
{
	double l = std::fabs(lattitude);
	if (l > M_HALF_PI)
		throw BadLattitude();

	if (l < 1.e-8) {
		_lattGeoDet = _lattGeoCen = lattitude;
		_pos = CartesianVector(0., 0., lattitude<0. ? -polarR : +polarR);
		_pos *= 1. + _altitude/_pos.norm();
		return;
	}
		
	if (lattType == GEODETIC) {
		_lattGeoDet = lattitude;
		_lattGeoCen = std::atan(polarRsquare/equatRsquare *
								std::tan(lattitude));
	} else {
		_lattGeoCen = lattitude;
		_lattGeoDet = std::atan(equatRsquare/polarRsquare *
								std::tan(lattitude));
	}

	double tanGeoLat = std::tan(_lattGeoCen),
		   R = sqrt(polarRsquare*equatRsquare/
						(polarRsquare + equatRsquare*tanGeoLat*tanGeoLat)),
		   z = R * tanGeoLat;

	_pos = CartesianVector(R * std::cos(_long), R * std::sin(_long), z);
	_pos *= 1. + _altitude/_pos.norm();
} // EarthObject::EarthObject()


//
//	---( EarthPositionLocator )------------------------------------------------
//
StateVector EarthPositionLocator::stateVector(double time) const throw()
{
	STime t = baseTime() + time;
	return stateVector(t);
} // EarthPositionLocator::stateVector()


StateVector EarthPositionLocator::stateVector(const STime &time) const throw()
{
	double st = time.siderealTime(_observer.longitude());
	return StateVector(_rotdist*std::cos(st), _rotdist*std::sin(st),
					   _observer.position().z());
} // EarthPositionLocator::stateVector()
