//-----------------------------------------------------------------------------
//
//						XMM Science Analysis System (SAS)
//					   (c) 1997-2010 European Space Agency
//
//-----------------------------------------------------------------------------
//
//	OdfDesignators.cc --- implementation of OdfDesignators
//
//	DESCRIPTION:
//		Implementation part of OdfDesignator class methods
//
//	CHANGE HISTORY:
//		UL, 2001-12-06: 1.0.0: - first version; partially taken from Odf.cc
//
//	AUTHOR:
//		U. Lammers, Mon Dec 15 16:30:06 MET 1997
//
//	REFERENCES:
//		[1]: Software Specification Document for the XMM Science Analysis
//			 Subsystem, M. Dahlem et. al.,
//		[2]: Observation Data File Interface Control Document, K. Galloway
//
//	$Header: /sasbuild/repositories/sasdev/sas/packages/caloalutils/src/OdfDesignators.cc,v 1.10 2004/08/04 12:18:14 rsaxton Exp $
//
#include <sstream>
#include <cstdio>	//	sscanf

#include "error.h"
#include "ExceptionErrorHandler.h"
#include "stringutil.h"
#include "OdfDesignators.h"
#include "StringIntMap.h"

using namespace std;

OdfSet::OdfSet() : _basename(""), _dirname("") {}

OdfSet::OdfSet(const string &name)
{
	Filename::StringPair p(Filename::split(name));
	_dirname = p.first;
	_basename = p.second;

	string _dstr[] = { // EMOS
					   "IME", "TIE", "RIE", "CTE", "AUX", "DII", "OVE",
					   "CCX", "PEH", "HCH", "HBH", "HTH",

					   "ECH",

					   "TMH", "PTH",

					   //	EPN not in above list
					   "BUE", "ODI", "HDI", "NOI", "DLI", "PMH", "PAH",

					   //	ERM not in above list
					   "ECE", "ESE",

					   //	RGS not in above list
					   "SPE", "HTE", "PFH", "PCH", "DPH", "D1H", "D2H", "OFX",

					   //	OM not in above list
					   "FAE", "E1I", "E2I", "E3I", "E4I", "E5I", "E6I", "E7I",
					   "PAX", "WDX", "PFX", "THX", "RFX", "NPH",

					   //	SC not in above list
					   "ATS", "RAS", "DAS", "POS", "ROS", "TCS", "TCX",
					   "P1S", "P2S", "P3S", "P4S", "P5S", "P6S",
					   "P7S", "P8S", "P9S", "SUM", "" };
	StringIntMap<Xmm::DataMode> dataMode(_dstr);

	string _fstr[] = { "E", "I", "X", "H", "S", "M", "" };
	StringIntMap<OdfSet::Type> fileType(_fstr);

	string _fmstr[] = { "FTZ", "ASZ", "FIT", "ASC", "SAS", "" };
	StringIntMap<OdfSet::Format> format(_fmstr);

	//	dissect the file name into fields
	char proposalId_cstr[11], instr_cstr[3], sflag, mode_cstr[4],
		 type_cstr[2], form_cstr[4];
	_nodeNr = 0;
	_isValid = false;
	unsigned nFields =
		sscanf(_basename.c_str(), "%4u_%[^_]_%2s%c%3u%2u%3s.%3s",
			   &_revolutionNr, proposalId_cstr, instr_cstr, &sflag,
			   &_exposureNr, &_ccdNr, mode_cstr, form_cstr);

	type_cstr[0] = mode_cstr[2]; type_cstr[1] = '\0';
	if (nFields != 8)
		errstr<<"`"<<_basename<<"' is not a valid ODF set name"
			  <<Sas::error("InvalidSetName");

	_proposalId = proposalId_cstr;
	if (sflag != 'S' && sflag != 'U' && sflag != 'X')
		errstr<<"The file named `"<<_basename<<"' does not belong to an ODF: "
				"the schedule flag field must be either 'S' or 'U' or 'X' but "
				"it is `"<<sflag
			  <<Sas::error("IllegalValue");

	_scheduled	= sflag == 'S';
	_scheduleFlagApplicable = sflag != 'X';
	_instr		= Xmm::toInstrument(instr_cstr);
	if (_instr == Xmm::EMOS1 || _instr == Xmm::EMOS2) {
		//	_ccdNr contains CCD and node number
		_nodeNr = _ccdNr % 10;
		_ccdNr = _ccdNr/10;
	}
	if (string(mode_cstr) != "IMI")
		_dataMode = dataMode[mode_cstr];
	else
		_dataMode = Xmm::IMAGING;
	_type		= fileType[type_cstr];
	_format		= format[form_cstr];

	_isValid = _instr != Xmm::INSTRUMENT_NONE &&
			   _dataMode != Xmm::DATA_MODE_NONE &&
			   _type != OdfSet::TNONE && _format != OdfSet::FNONE;
} // OdfSet::OdfSet()

const string & OdfSet::proposalId () const  { return _proposalId; }

bool OdfSet::isValid(const std::string &name)
{
	ExceptionErrorHandler local;
	ErrorHandler *oldHandler = errHandler.handler(&local);

	bool valid = false;
	try {
		OdfSet odfSet(name);
		valid = odfSet.isValid();
	} catch(...) {}

	//  restore old error handler
    errHandler.handler(oldHandler);

	return valid;
} // OdfSet::isValid()


ostream &operator<<(ostream &os, const OdfSet &fname)
{
	os<<"basename          : "<<fname.basename()<<endl
	  <<"dirname           : "<<fname.dirname()<<endl
	  <<"revolution number : "<<fname.revolutionNr()<<endl
	  <<"proposal Id       : "<<fname.proposalId()<<endl
	  <<"instrument        : "<<Xmm::toString(fname.instrument())<<endl
	  <<"data mode         : "<<Xmm::toString(fname.dataMode())<<endl
	  <<"exposure number   : "<<fname.exposureNr()<<endl
	  <<"scheduled         : "<<(fname.scheduled() ? "yes" : "no")<<endl
	  <<"CCD/window number : "<<fname.ccdNr()<<endl
	  <<"node number       : "<<fname.nodeNr()<<endl
	  <<"type              : "<<UNT(fname.type())<<endl
	  <<"format            : "<<UNT(fname.format())<<endl;

	return os;
} // operator<<()


InstrumentExposureIdentifier::InstrumentExposureIdentifier(const string &id)
	: _instr(Xmm::INSTRUMENT_NONE), _scheduleFlagApplicable(false),
	  _scheduled(false), _expNr(0)
{
	if (id.length() > 6) {
		//	assume this is an ODF set name
		OdfSet odfset(id);
		_instr = odfset.instrument();
		_scheduleFlagApplicable = odfset.scheduleFlagApplicable();
		_scheduled = odfset.scheduled();
		_expNr = odfset.exposureNr();
	} else if (id.length() != 6)
		errstr<<"Given instr/exposure designator `"<<id
			  <<"' does not match expected pattern "
				"`{M1}|{M2}|{PN}{R1}{R2}{OM}S|U[0-9][0-9][0-9]'"
			  <<Sas::error("InvalidInstrumentExposureDesignator");
	else {
		_instr = Xmm::toInstrument(id.substr(0, 2));
		_scheduleFlagApplicable = id[2] != 'X';
		_scheduled = _scheduleFlagApplicable && id[2] == 'S';
		_expNr = toNumber<unsigned>(id.substr(3));
	}
} // InstrumentExposureIdentifier::InstrumentExposureIdentifier()


string InstrumentExposureIdentifier::exposureId() const
{
	stringstream s;
	s<<(_scheduleFlagApplicable ? (_scheduled ? 'S' : 'U') : 'X')
	 <<setw(3)<<setfill('0')<<_expNr;

	return s.str();
} // InstrumentExposureIdentifier::exposureId()


string InstrumentExposureIdentifier::instrumentExposureId() const
{
	return Xmm::toString(_instr, true) + exposureId();
} // InstrumentExposureIdentifier::instrumentExposureId()
